using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using NaGet.Tasks;

namespace NaGet.SubCommands
{	
	#region NaGetTaskQueryHandler֘A
	
	public delegate NaGetTaskQueryResult NaGetTaskQueryHandler(object sender, NaGetTaskQueryArgs e);
	
	public class NaGetTaskQueryArgs
	{
		/// <summary>
		/// ^XŇ̃bZ[W
		/// </summary>
		public string Message;
		
		/// <summary>
		/// 񓚂̎ނ̃tO
		/// </summary>
		public NaGetTaskQueryResult SelectionFlag;
		
		public NaGetTaskQueryArgs(string message, NaGetTaskQueryResult selectionFlag)
		{
			this.Message = message;
			this.SelectionFlag = selectionFlag;
		}
	}
	
	/// <summary>
	/// ̓
	/// </summary>
	[Flags()]
	public enum NaGetTaskQueryResult : uint
	{
		/// <summary>
		/// p(=OK)
		/// </summary>
		CONTINUE = 0x01,
		/// <summary>
		/// Ďs
		/// </summary>
		RETRY = 0x02,
		/// <summary>
		/// LZA~
		/// </summary>
		CANCEL = 0x04,
		/// <summary>
		/// IȃLZ([U̎ł͂Ȃ)
		/// </summary>
		CANCELED_AUTOMATICALLY = 0x00,
	}
	
	
	#endregion
	
	/// <summary>
	/// NaGet^XÑpbN
	/// </summary>
	public abstract class NaGetTaskSet : Task
	{
		/// <summary>
		/// ^XN̎̃nh
		/// </summary>
		public event NaGetTaskQueryHandler TaskQueryRaised;
		
		/// <summary>
		/// ŕ\ƈꗗXg
		/// </summary>
		protected IList<string> taskSetNames;
		
		/// <summary>
		/// ŕ\ƈꗗXg
		/// </summary>
		public virtual IList<string> TaskSetNames {
			get {
				return new ReadOnlyCollection<string>(taskSetNames);
			}
		}
		
		/// <summary>
		/// ݎs̍Ɣԍ
		/// </summary>
		public abstract int CurrentTaskSetIndex {
			get;
		}
		
		/// <summary>
		/// ݂̐i߂B
		/// </summary>
		/// <param name="type">Ƃ̏</param>
		/// <param name="subTaskProgress">Tu^XN̐i</param>
		/// <returns>݂̐i</returns>
		protected virtual float GetProgressPercent(TaskEventType type, float subTaskProgress)
		{
			if (CurrentTaskSetIndex >= 0) {
				if (subTaskProgress >= 0) {
					return (CurrentTaskSetIndex * 100 + subTaskProgress) / taskSetNames.Count;
				}
				switch (type) {
					case TaskEventType.STARTED:
						return 0;
					case TaskEventType.COMPLETED:
						return 100;
					case TaskEventType.COMPLETED_SUBTASK:
						return ((CurrentTaskSetIndex+1) * 100) / taskSetNames.Count;
					default:
						return (CurrentTaskSetIndex * 100) / taskSetNames.Count;
				}
			}
			
			return -1;
		}
		
		#region TaskEvent֗\bh
		
		protected virtual void RaiseTaskSetEvent(TaskEventType type, string message)
		{
			RaiseTaskSetEvent(type, message, GetProgressPercent(type, -1));
		}
		
		protected virtual void ReceivedErrorData(object sender, NaGet.Utils.AnyDataEventArgs<string> e)
		{
			if (! string.IsNullOrEmpty(e.Data)) {
				RaiseTaskSetEvent(TaskEventType.WARNING, e.Data);
			}
		}
		
		protected virtual void ReceivedOutputData(object sender, NaGet.Utils.AnyDataEventArgs<string> e)
		{
			if (! string.IsNullOrEmpty(e.Data)) {
				RaiseTaskSetEvent(TaskEventType.INFO, e.Data);
			}
		}
		
		#endregion
		
		protected virtual NaGetTaskQueryResult RaiseTaskSetQueryEvent(string message, NaGetTaskQueryResult selection)
		{
			if (TaskQueryRaised != null) {
				return TaskQueryRaised(this, new NaGetTaskQueryArgs(message, selection));
			}
			return NaGetTaskQueryResult.CANCELED_AUTOMATICALLY;
		}
		
		public override bool Running {
			get { return CurrentTaskSetIndex >= 0 && !Done; }
		}
	}
}
